<?php

/**
 * @file
 * Administrative configuration for the Project Documentation module.
 */

/**
 * Implements hook_form
 */
function projectdocumentation_config_form($form, &$form_state) {
  $projects = array();
  $result = db_query("SELECT filename, name, info, bootstrap, schema_version, status FROM {system} WHERE type = 'module'");
  while ($module = $result->fetchObject()) {
    // Exclude non-contributed modules (path does not start with 's').
    if ('s' != $module->filename[0] ) {
      continue;
    }
    $info = unserialize($module->info);
    $shortname = $module->name;
    if (empty($info['project'])) {
      // Modules not from the Drupal.org repo are missing the 'project' key in *.info.
      $project = $shortname;
      $gotproject = FALSE;
    }
    else {
      $project = $info['project'];
      $gotproject = TRUE;
    }
    // Assumes that all projects have at least one module with the same name.
    if (!isset($projects[$project]) && ($project == $shortname)) {
      $module->longname = $info['name'];
      $module->description = $info['description'];
      $module->date = date('Y-m-d h:i', $info['mtime']);
      $module->shortname = $shortname;
      $module->longinfo = $info;
      $module->gotproject = $gotproject;
      $modarray[$info['name']] = $module;
    }
  }
  ksort($modarray, SORT_STRING | SORT_FLAG_CASE);
  $markup = '<h2>Summary</h2><ol>';
  $create = $missing = $done = 0;
  foreach ($modarray as $module) {
    $status = _projectdocumentation_status($module->shortname);
    $query = new EntityFieldQuery();
    $query->entityCondition('entity_type', 'node')
      ->propertyCondition('type', 'projectdocumentation')
      ->propertyCondition('title', $module->longname);
    $result = $query->execute();
    if (empty($result['node'])) {
      if ($module->gotproject) {
        $markup .= '<li><strong>' . t('Project “@longname”</strong> - <code>!shortname</code> (!status) <strong>can be added to the project list.', array('@longname' => $module->longname, '!shortname' => $module->shortname, '!status' => $status)) . '</strong></li>';
	$form_state['pd']['cancreate'][] = $module;
	$create++;
      }
      else {
        $markup .= '<li>' . t('File <code>@shortname.info</code> for “@longname” is missing the “project” key.', array('@shortname' => $module->shortname, '@longname' => $module->longname)) . '</li>';
	$missing++;
      }
    }
    else {
      $markup .= '<li>' . t('Project “@longname” - <code>!shortname</code> (!status) is alredy documented.', array('@longname' => $module->longname, '!shortname' => $module->shortname, '!status' => $status)) . '</li>';
      $done++;
    }
  }
  $markup .= '</ol><p>';
  $markup .= t('!create project(s) can be documented. !missing project(s) missing “project” key. !done project(s) already documented.', array('!create' => $create, '!missing' => $missing, '!done' => $done));
  $markup .= '</p>';
  if ($missing) {
    $markup .= '<p>Projects where the “project” key is missing has not been packaged by Drupal.org. They are usually custom projects that has been added locally. You can add them to the project list manually.</p>';
  }
  $form['pd']['projectlist'] = array(
    '#markup' => $markup,
  );
  $form['datetimep'] = array(
      '#type' =>'checkbox',
      '#title'=>t("Fill in datetime field with today's date and time."),
      '#description'=>t("If not checked,  the node's date field will be empty."),
      '#default_value' =>TRUE,
  );

  if ($create) {
    $form['actions']['submit'] = array(
      '#type' => 'submit',
      '#value' => t('Create documentation for !projects', array('!projects' => format_plural($create, 'one project', '@count projects'))),
    );
  }
 if (0 == $create) {
    $form['actions']['submit']['#disabled'] = TRUE;
  }
  $form['actions']['cancel'] = array(
    '#type'   => 'button',
    '#value' => t('Cancel'),
    '#executes_submit_callback' => TRUE,
    '#submit' => array('_projectdocumentation_form_cancel'),
  );
  return $form;
}

/**
 * Submission handler.
 */
function projectdocumentation_config_form_submit($form, &$form_state) {
  foreach ($form_state['pd']['cancreate'] as $module) {
    _projectdocumentation_createnode($module, $form_state['values']['datetimep']);
  }
}

/**
 * Cancel handler.
 */
function _projectdocumentation_form_cancel(){
  drupal_goto('/admin/config/system');
}

function _projectdocumentation_createnode($module, $dtp) {
global $user;
  $values = array(
    'type' => 'projectdocumentation',
    'uid' => $user->uid,
    'status' => 1,
    'comment' => 1,
    'promote' => 0,
  );
  $entity = entity_create('node', $values);
  $ewrapper = entity_metadata_wrapper('node', $entity);
  $ewrapper->title->set($module->longname);
  $ewrapper->body->set(array('value' => 'Explain why this project was installed.'));
  $ewrapper->field_projdoc_shortname->set($module->shortname);
  if ($dtp) {
    $ewrapper->field_projdoc_dateinst->set(REQUEST_TIME);
  }

  // Entity API cannot set some values so the old method must be used
  $entity->field_projdoc_projectpage[LANGUAGE_NONE][0]['url'] = 'https://www.drupal.org/project/' . $module->shortname;
  $entity->field_projdoc_projectpage[LANGUAGE_NONE][0]['attributes'] = array();

  $ewrapper->save();
  drupal_set_message(t('Adding <strong>!module</strong> to project list.', array('!module' => $module->longname)));
}
